RengarusStatusDisplay = {}
RengarusStatusDisplay.name = "RengarusStatusDisplay"
RengarusStatusDisplay.version="1"

local LAM2 = LibAddonMenu2 or false

RengarusStatusDisplay.Default = {
	offsetX = 0,
	offsetY = -60,
	
	scale = 1,

	toggleStunned = true,
	toggleFlash = true,
	toggleStaggered = true,
	toggleRooted = true,
	toggleSilenced = true,
	toggleDodging = true,
	toggleBlocking = true,
	toggleSprinting = true,
	toggleTips = true,
	
	stringStunned = "STUNNED",
	stringScared = "SCARED",
	stringUnbreakable = "UNBREAKABLE",
	stringStaggered = "STAGGERED",
	stringRooted = "ROOTED",
	stringSilenced = "SILENCED",
	stringDodging = "DODGING",
	stringBlocking = "BLOCKING",
	stringSprinting = "SPRINTING",
	stringVampireSprinting = "STEALTHED",
	
	colorStunnedR = 1,
	colorStunnedG = 0,
	colorStunnedB = 0,
	colorStunnedA = 1,
	
	colorStaggeredR = 0.5,
	colorStaggeredG = 0.5,
	colorStaggeredB = 0.5,
	colorStaggeredA = 1,
	
	colorRootedR = 1,
	colorRootedG = 1,
	colorRootedB = 0,
	colorRootedA = 1,
	
	colorSilencedR = 0.75,
	colorSilencedG = 0,
	colorSilencedB = 0.75,
	colorSilencedA = 1,

	colorDodgingR = 0,
	colorDodgingB = 1,
	colorDodgingG = 1,
	colorDodgingA = 1,
	
	colorBlockingR = 0.25,
	colorBlockingG = 0.5,
	colorBlockingB = 1,
	colorBlockingA = 1,

	colorSprintingR = 0,
	colorSprintingB = 0,
	colorSprintingG = 1,
	colorSprintingA = 1,
}

local isBreakable = false
local isStunned = false
local isScared = false
local isStaggered = false
local isRooted = false
local isSilenced = false
local isBlocking = false
local isDodging = false
local isSprinting = false
local isVampireSprinting = false

local durationStaggered = 0

local lastPowerValue = 0
local currentPowerValue = 0
local lastPowerDrain = GetFrameTimeSeconds()

-----------------
-- Identifiers --
-----------------

local function IsSilence(abilityId)
	--Icon
	if GetAbilityIcon(abilityId) == "/esoui/art/icons/ability_debuff_silence.dds" then
		return true
	end
	--Silence Effect
	if abilityId == 6289 then
		return true
	end
	--NPC Negate Magic
	if abilityId == 80735 then
		return true
	end
	--Negate Magic
	if abilityId == 27706 or abilityId == 28341 or abilityId == 28348 then
		return true
	end
end

local function IsIgnore(abilityId)
	--Hiding Spot
	if abilityId == 72712 or abilityId == 75747 or abilityId == 75860 or abilityId == 126609 then
		return true
	end
	--Enter the Endless
	if abilityId == 194570 or abilityId == 194571 or abilityId == 202141 or abilityId == 202803 or abilityId == 212065 then
		return true
	end
	--Verse Select
	if abilityId == 203124 or abilityId == 203125 or abilityId ==  210189 then
		return true
	end
	--Vision Select
	if abilityId == 202995 or abilityId == 203101 then
		return true
	end
	--Side Content Transporter
	if abilityId == 211431 or abilityId == 211433 or abilityId == 211440 then
		return true
	end
end

---------------
-- Functions --
---------------

local function UpdateText()
	if not IsPlayerStunned() then
		isStunned = false
	end
	
	if IsPlayerTryingToMove() and IsPlayerMoving() then
		isRooted = false
		durationStaggered = 0
	end

	if durationStaggered > GetFrameTimeSeconds() then
		isStaggered = true
	else
		isStaggered = false
	end

	if (isStunned or isScared) and RengarusStatusDisplay.savedVariables.toggleFlash then
		FlashHealthWarningStage(2,1000)
	end
	
	if RengarusStatusDisplay.savedVariables.toggleTips then
		ZO_ActiveCombatTips:SetHidden(true)
	else
		ZO_ActiveCombatTips:SetHidden(false)
	end

	if isRooted and isSilenced and RengarusStatusDisplay.savedVariables.toggleRooted and RengarusStatusDisplay.savedVariables.toggleSilenced then
		RengarusStatusDisplayTextUP:SetHidden(false)
		RengarusStatusDisplayTextUPLabel:SetColor(RengarusStatusDisplay.savedVariables.colorRootedR,RengarusStatusDisplay.savedVariables.colorRootedG,RengarusStatusDisplay.savedVariables.colorRootedB,RengarusStatusDisplay.savedVariables.colorRootedA)
		RengarusStatusDisplayTextUPLabel:SetText(RengarusStatusDisplay.savedVariables.stringRooted.." & "..RengarusStatusDisplay.savedVariables.stringSilenced)
	elseif isRooted and RengarusStatusDisplay.savedVariables.toggleRooted then
		RengarusStatusDisplayTextUP:SetHidden(false)
		RengarusStatusDisplayTextUPLabel:SetColor(RengarusStatusDisplay.savedVariables.colorRootedR,RengarusStatusDisplay.savedVariables.colorRootedG,RengarusStatusDisplay.savedVariables.colorRootedB,RengarusStatusDisplay.savedVariables.colorRootedA)
		RengarusStatusDisplayTextUPLabel:SetText(RengarusStatusDisplay.savedVariables.stringRooted)
	elseif isSilenced and RengarusStatusDisplay.savedVariables.toggleSilenced then
		RengarusStatusDisplayTextUP:SetHidden(false)
		RengarusStatusDisplayTextUPLabel:SetColor(RengarusStatusDisplay.savedVariables.colorSilencedR,RengarusStatusDisplay.savedVariables.colorSilencedG,RengarusStatusDisplay.savedVariables.colorSilencedB,RengarusStatusDisplay.savedVariables.colorSilencedA)
		RengarusStatusDisplayTextUPLabel:SetText(RengarusStatusDisplay.savedVariables.stringSilenced)
	else
		RengarusStatusDisplayTextUP:SetHidden(true)
	end
	
	if isScared and RengarusStatusDisplay.savedVariables.toggleStunned then
		if not isBreakable then
			RengarusStatusDisplayTextUP:SetHidden(false)
			RengarusStatusDisplayTextUPLabel:SetColor(RengarusStatusDisplay.savedVariables.colorStunnedR,RengarusStatusDisplay.savedVariables.colorStunnedG,RengarusStatusDisplay.savedVariables.colorStunnedB,RengarusStatusDisplay.savedVariables.colorStunnedA)
			RengarusStatusDisplayTextUPLabel:SetText(RengarusStatusDisplay.savedVariables.stringUnbreakable)
		end
		RengarusStatusDisplayTextDOWN:SetHidden(false)
		RengarusStatusDisplayTextDOWNLabel:SetColor(RengarusStatusDisplay.savedVariables.colorStunnedR,RengarusStatusDisplay.savedVariables.colorStunnedG,RengarusStatusDisplay.savedVariables.colorStunnedB,RengarusStatusDisplay.savedVariables.colorStunnedA)
		RengarusStatusDisplayTextDOWNLabel:SetText(RengarusStatusDisplay.savedVariables.stringScared)
	elseif isStunned and RengarusStatusDisplay.savedVariables.toggleStunned then
		if not isBreakable then
			RengarusStatusDisplayTextUP:SetHidden(false)
			RengarusStatusDisplayTextUPLabel:SetColor(RengarusStatusDisplay.savedVariables.colorStunnedR,RengarusStatusDisplay.savedVariables.colorStunnedG,RengarusStatusDisplay.savedVariables.colorStunnedB,RengarusStatusDisplay.savedVariables.colorStunnedA)
			RengarusStatusDisplayTextUPLabel:SetText(RengarusStatusDisplay.savedVariables.stringUnbreakable)
		end
		RengarusStatusDisplayTextDOWN:SetHidden(false)
		RengarusStatusDisplayTextDOWNLabel:SetColor(RengarusStatusDisplay.savedVariables.colorStunnedR,RengarusStatusDisplay.savedVariables.colorStunnedG,RengarusStatusDisplay.savedVariables.colorStunnedB,RengarusStatusDisplay.savedVariables.colorStunnedA)
		RengarusStatusDisplayTextDOWNLabel:SetText(RengarusStatusDisplay.savedVariables.stringStunned)
	elseif isStaggered and RengarusStatusDisplay.savedVariables.toggleStaggered then
		RengarusStatusDisplayTextDOWN:SetHidden(false)
		RengarusStatusDisplayTextDOWNLabel:SetColor(RengarusStatusDisplay.savedVariables.colorStaggeredR,RengarusStatusDisplay.savedVariables.colorStaggeredG,RengarusStatusDisplay.savedVariables.colorStaggeredB,RengarusStatusDisplay.savedVariables.colorStaggeredA)
		RengarusStatusDisplayTextDOWNLabel:SetText(RengarusStatusDisplay.savedVariables.stringStaggered)
	elseif isDodging and RengarusStatusDisplay.savedVariables.toggleDodging then
		RengarusStatusDisplayTextDOWN:SetHidden(false)
		RengarusStatusDisplayTextDOWNLabel:SetColor(RengarusStatusDisplay.savedVariables.colorDodgingR,RengarusStatusDisplay.savedVariables.colorDodgingG,RengarusStatusDisplay.savedVariables.colorDodgingB,RengarusStatusDisplay.savedVariables.colorDodgingA)
		RengarusStatusDisplayTextDOWNLabel:SetText(RengarusStatusDisplay.savedVariables.stringDodging)
	elseif isBlocking and not (isScared or isStunned or isStaggered or isSprinting or isDodging) and RengarusStatusDisplay.savedVariables.toggleBlocking then
		RengarusStatusDisplayTextDOWN:SetHidden(false)
		RengarusStatusDisplayTextDOWNLabel:SetColor(RengarusStatusDisplay.savedVariables.colorBlockingR,RengarusStatusDisplay.savedVariables.colorBlockingG,RengarusStatusDisplay.savedVariables.colorBlockingB,RengarusStatusDisplay.savedVariables.colorBlockingA)
		RengarusStatusDisplayTextDOWNLabel:SetText(RengarusStatusDisplay.savedVariables.stringBlocking)
	elseif isSprinting and RengarusStatusDisplay.savedVariables.toggleSprinting then
		if isVampireSprinting then
			RengarusStatusDisplayTextUP:SetHidden(false)
			RengarusStatusDisplayTextUPLabel:SetColor(RengarusStatusDisplay.savedVariables.colorSprintingR,RengarusStatusDisplay.savedVariables.colorSprintingG,RengarusStatusDisplay.savedVariables.colorSprintingB,RengarusStatusDisplay.savedVariables.colorSprintingA)
			RengarusStatusDisplayTextUPLabel:SetText(RengarusStatusDisplay.savedVariables.stringVampireSprinting)
		end
		RengarusStatusDisplayTextDOWN:SetHidden(false)
		RengarusStatusDisplayTextDOWNLabel:SetColor(RengarusStatusDisplay.savedVariables.colorSprintingR,RengarusStatusDisplay.savedVariables.colorSprintingG,RengarusStatusDisplay.savedVariables.colorSprintingB,RengarusStatusDisplay.savedVariables.colorSprintingA)
		RengarusStatusDisplayTextDOWNLabel:SetText(RengarusStatusDisplay.savedVariables.stringSprinting)
	else
		RengarusStatusDisplayTextDOWN:SetHidden(true)
	end
end

local function DisplayTip(event, tipType)
	--d("display",tipType)
	if tipType == 18 then
		isBreakable = true
		UpdateText()
	end
	if tipType == 19 then 
		isRooted = true
		UpdateText()
	end
end

local function RemoveTip(event, tipType, result)
	--d("remove",tipType)
	if tipType == 18 then
		isBreakable = false
		isStunned = false
		isScared = false
		UpdateText()
	end
	if tipType == 19 then
		isRooted = false
		UpdateText()
	end
end

local function OnEffectChanged(eventCode, changeType, effectSlot, effectName, unitTag, beginTime, endTime, stackCount, iconName, buffType, effectType, abilityType, statusEffectType, unitName, unitId, abilityId, sourceType)
	if unitTag == "player" then
		if IsSilence(abilityId) then
			if changeType == 1 then
				isSilenced = true
				UpdateText()
			end
			if changeType == 2 then
				isSilenced = false
				UpdateText()
			end
		elseif abilityId == 135226 then
			if changeType == 1 then
				isVampireSprinting = true
			end
			if changeType == 2 then
				isVampireSprinting = false
			end
		end
		if changeType == 1 and abilityId == 29721 then
			isDodging = true
			zo_callLater(function () isDodging = false end, 1000)
			UpdateText()
			zo_callLater(function () UpdateText() end, 1050)
		end
	end
end

local function OnCombatEvent(eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log, sourceUnitId, targetUnitId, abilityId)
	if targetName == GetRawUnitName("player") and targetType == 1 and sourceType ~= 1 and not IsIgnore(abilityId) then
		if result == ACTION_RESULT_STUNNED and GetAbilityDuration(abilityId) > 0 then
			--d(abilityName,abilityId,sourceName,sourceType)
			isStunned = true
			zo_callLater(function () isStunned = false end, GetAbilityDuration(abilityId))
			UpdateText()
			zo_callLater(function () UpdateText() end, GetAbilityDuration(abilityId) + 50)
		end
		if result == ACTION_RESULT_FEARED and GetAbilityDuration(abilityId) > 0 then
			--d(abilityName,abilityId,sourceName,sourceType)
			isScared = true
			zo_callLater(function () isScared = false end, GetAbilityDuration(abilityId))
			UpdateText()
			zo_callLater(function () UpdateText() end, GetAbilityDuration(abilityId) + 50)
		end
		if result == ACTION_RESULT_STAGGERED and GetAbilityDuration(abilityId) > 0 then
			--d(abilityName,abilityId,sourceName,sourceType)
			durationStaggered = math.max(durationStaggered,GetFrameTimeSeconds() + GetAbilityDuration(abilityId) / 1000)
			UpdateText()
			zo_callLater(function () UpdateText() end, GetAbilityDuration(abilityId) + 50)
		end
	end
end

local function CheckBlocking()
	if IsBlockActive() and (GetPlayerStat(STAT_STAMINA_REGEN_COMBAT, STAT_BONUS_OPTION_APPLY_BONUS) == 0 or GetPlayerStat(STAT_MAGICKA_REGEN_COMBAT, STAT_BONUS_OPTION_APPLY_BONUS) == 0) then
		isBlocking = true
		zo_callLater(function () CheckBlocking() end, 50)
	else
		isBlocking = false
	end
	UpdateText()
end

local function CheckSprinting()
	if lastPowerDrain + 0.1 > GetFrameTimeSeconds() then
		isSprinting = true
		zo_callLater(function () CheckSprinting() end, 50)
	else
		isSprinting = false
	end
	UpdateText()
end

local function OnPowerUpdate(eventCode, unitTag, powerIndex, powerType, powerValue, powerMax, powerEffectiveMax)
	if unitTag == "player" then
		if (powerIndex == 1 or powerIndex == 3 or powerIndex == 5) and (GetUnitStealthState("player") == 0 or isVampireSprinting) then
			lastPowerValue = currentPowerValue
			currentPowerValue = powerValue
			if (lastPowerValue - currentPowerValue) < 50 and (lastPowerValue - currentPowerValue) > 0 then
				lastPowerDrain = GetFrameTimeSeconds()
				if isSprinting == false then
					CheckSprinting()
				end
			end
		end
		if (powerIndex == 1 or powerIndex == 3) and isBlocking == false then
			CheckBlocking()
		end
	end
end

local function UpdateAnchors()
	RengarusStatusDisplayTextUP:SetScale(RengarusStatusDisplay.savedVariables.scale)
	RengarusStatusDisplayTextUP:ClearAnchors()
	RengarusStatusDisplayTextUP:SetAnchor(CENTER, GuiRoot, CENTER, RengarusStatusDisplay.savedVariables.offsetX, (RengarusStatusDisplay.savedVariables.offsetY - (30 * RengarusStatusDisplay.savedVariables.scale)))
	RengarusStatusDisplayTextDOWN:SetScale(RengarusStatusDisplay.savedVariables.scale)
	RengarusStatusDisplayTextDOWN:ClearAnchors()
	RengarusStatusDisplayTextDOWN:SetAnchor(CENTER, GuiRoot, CENTER, RengarusStatusDisplay.savedVariables.offsetX, RengarusStatusDisplay.savedVariables.offsetY)
end

local function PreviewPosition()
	RengarusStatusDisplayTextUP:SetHidden(false)
	RengarusStatusDisplayTextUPLabel:SetColor(1,1,1,1)
	RengarusStatusDisplayTextUPLabel:SetText("TEST")
	RengarusStatusDisplayTextDOWN:SetHidden(false)
	RengarusStatusDisplayTextDOWNLabel:SetColor(1,1,1,1)
	RengarusStatusDisplayTextDOWNLabel:SetText("PREVIEW")
end

---------------------
-- Settings Window --
---------------------

local function CreateSettingsWindow()
	local panelData = {
		type = "panel",
		name = "Rengaru's Status Display",
		displayName = "Rengaru's Status Display",
		author = "Rengaru",
		version = RengarusStatusDisplay.version,
		registerForRefresh = true,
		registerForDefaults = true,
	} 
	local cntrlOptionsPanel = LAM2:RegisterAddonPanel("RengarusStatusDisplay_SettingsPanel", panelData)
	local optionsData = {
		--Stunned
		[1] = {
			type = "header",
			name = "Stunned & Scared",
		},
		[2] = {
			type = "checkbox",
			name = "Toggle:",
			tooltip = "Show when you are stunned or scared.\nTo properly show when a stun is unbreakable Active Combat Tips in the Combat Settings must be set to Always Show.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleStunned end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleStunned = newValue end,
			width = "half",
		},
		[3] = {
		  type = "colorpicker",
		  name = "Color:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleStunned end,
		  default = {r = RengarusStatusDisplay.Default.colorStunnedR,
					 g = RengarusStatusDisplay.Default.colorStunnedG,
					 b = RengarusStatusDisplay.Default.colorStunnedB,
					 a = RengarusStatusDisplay.Default.colorStunnedA},
		  getFunc = function() return
						RengarusStatusDisplay.savedVariables.colorStunnedR,
						RengarusStatusDisplay.savedVariables.colorStunnedG,
						RengarusStatusDisplay.savedVariables.colorStunnedB,
						RengarusStatusDisplay.savedVariables.colorStunnedA
					end,
		  setFunc = function(r,g,b,a)
						RengarusStatusDisplay.savedVariables.colorStunnedR = r
						RengarusStatusDisplay.savedVariables.colorStunnedG = g
						RengarusStatusDisplay.savedVariables.colorStunnedB = b
						RengarusStatusDisplay.savedVariables.colorStunnedA = a
					end,
		  width = "half",
		},
		[4] = {
		  type = "editbox",
		  name = "Text Stunned:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleStunned end,
		  default = RengarusStatusDisplay.Default.stringStunned,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringStunned end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringStunned = text end,
		  isMultiline = false,
		  width = "half",
		},
		[5] = {
		  type = "editbox",
		  name = "Text Scared:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleStunned end,
		  default = RengarusStatusDisplay.Default.stringScared,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringScared end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringScared = text end,
		  isMultiline = false,
		  width = "half",
		},
		[6] = {
		  type = "editbox",
		  name = "Text Unbreakable:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleStunned end,
		  default = RengarusStatusDisplay.Default.stringUnbreakable,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringUnbreakable end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringUnbreakable = text end,
		  isMultiline = false,
		  width = "half",
		},
		[7] = {
			type = "checkbox",
			name = "Flash Screen",
			tooltip = "Flashes the Screen when you get stunned or scared.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleFlash end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleFlash = newValue end,
			width = "half",
		},
		--Staggered
		[8] = {
			type = "header",
			name = "Staggered",
		},		
		[9] = {
			type = "checkbox",
			name = "Toggle:",
			tooltip = "Show when you are staggered.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleStaggered end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleStaggered = newValue end,
			width = "half",
		},
		[10] = {
		  type = "colorpicker",
		  name = "Color:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleStaggered end,
		  default = {r = RengarusStatusDisplay.Default.colorStaggeredR,
					 g = RengarusStatusDisplay.Default.colorStaggeredG,
					 b = RengarusStatusDisplay.Default.colorStaggeredB,
					 a = RengarusStatusDisplay.Default.colorStaggeredA},
		  getFunc = function() return
						RengarusStatusDisplay.savedVariables.colorStaggeredR,
						RengarusStatusDisplay.savedVariables.colorStaggeredG,
						RengarusStatusDisplay.savedVariables.colorStaggeredB,
						RengarusStatusDisplay.savedVariables.colorStaggeredA
					end,
		  setFunc = function(r,g,b,a)
						RengarusStatusDisplay.savedVariables.colorStaggeredR = r
						RengarusStatusDisplay.savedVariables.colorStaggeredG = g
						RengarusStatusDisplay.savedVariables.colorStaggeredB = b
						RengarusStatusDisplay.savedVariables.colorStaggeredA = a
					end,
		  width = "half",
		},
		[11] = {
		  type = "editbox",
		  name = "Text:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleStaggered end,
		  default = RengarusStatusDisplay.Default.stringStaggered,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringStaggered end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringStaggered = text end,
		  isMultiline = false,
		  width = "half",
		},
		--Rooted
		[12] = {
			type = "header",
			name = "Rooted",
		},
		[13] = {
			type = "checkbox",
			name = "Toggle:",
			tooltip = "Show when you are rooted.\nActive Combat Tips in the Combat Settings must be set to Always Show.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleRooted end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleRooted = newValue end,
			width = "half",
		},
		[14] = {
		  type = "colorpicker",
		  name = "Color:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleRooted end,
		  default = {r = RengarusStatusDisplay.Default.colorRootedR,
					 g = RengarusStatusDisplay.Default.colorRootedG,
					 b = RengarusStatusDisplay.Default.colorRootedB,
					 a = RengarusStatusDisplay.Default.colorRootedA},
		  getFunc = function() return
						RengarusStatusDisplay.savedVariables.colorRootedR,
						RengarusStatusDisplay.savedVariables.colorRootedG,
						RengarusStatusDisplay.savedVariables.colorRootedB,
						RengarusStatusDisplay.savedVariables.colorRootedA
					end,
		  setFunc = function(r,g,b,a)
						RengarusStatusDisplay.savedVariables.colorRootedR = r
						RengarusStatusDisplay.savedVariables.colorRootedG = g
						RengarusStatusDisplay.savedVariables.colorRootedB = b
						RengarusStatusDisplay.savedVariables.colorRootedA = a
					end,
		  width = "half",
		},
		[15] = {
		  type = "editbox",
		  name = "Text:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleRooted end,
		  default = RengarusStatusDisplay.Default.stringRooted,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringRooted end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringRooted = text end,
		  isMultiline = false,
		  width = "half",
		},
		--Silenced
		[16] = {
			type = "header",
			name = "Silenced",
		},
		[17] = {
			type = "checkbox",
			name = "Toggle:",
			tooltip = "Show when you are silenced.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleSilenced end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleSilenced = newValue end,
			width = "half",
		},
		[18] = {
		  type = "colorpicker",
		  name = "Color:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleSilenced end,
		  default = {r = RengarusStatusDisplay.Default.colorSilencedR,
					 g = RengarusStatusDisplay.Default.colorSilencedG,
					 b = RengarusStatusDisplay.Default.colorSilencedB,
					 a = RengarusStatusDisplay.Default.colorSilencedA},
		  getFunc = function() return
						RengarusStatusDisplay.savedVariables.colorSilencedR,
						RengarusStatusDisplay.savedVariables.colorSilencedG,
						RengarusStatusDisplay.savedVariables.colorSilencedB,
						RengarusStatusDisplay.savedVariables.colorSilencedA
					end,
		  setFunc = function(r,g,b,a)
						RengarusStatusDisplay.savedVariables.colorSilencedR = r
						RengarusStatusDisplay.savedVariables.colorSilencedG = g
						RengarusStatusDisplay.savedVariables.colorSilencedB = b
						RengarusStatusDisplay.savedVariables.colorSilencedA = a
					end,
		  width = "half",
		},
		[19] = {
		  type = "editbox",
		  name = "Text:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleSilenced end,
		  default = RengarusStatusDisplay.Default.stringSilenced,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringSilenced end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringSilenced = text end,
		  isMultiline = false,
		  width = "half",
		},
		--Dodging
		[20] = {
			type = "header",
			name = "Dodging",
		},
		[21] = {
			type = "checkbox",
			name = "Toggle:",
			tooltip = "Show when you are dodging.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleDodging end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleDodging = newValue end,
			width = "half",
		},
		[22] = {
		  type = "colorpicker",
		  name = "Color:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleDodging end,
		  default = {r = RengarusStatusDisplay.Default.colorDodgingR,
					 g = RengarusStatusDisplay.Default.colorDodgingG,
					 b = RengarusStatusDisplay.Default.colorDodgingB,
					 a = RengarusStatusDisplay.Default.colorDodgingA},
		  getFunc = function() return
						RengarusStatusDisplay.savedVariables.colorDodgingR,
						RengarusStatusDisplay.savedVariables.colorDodgingG,
						RengarusStatusDisplay.savedVariables.colorDodgingB,
						RengarusStatusDisplay.savedVariables.colorDodgingA
					end,
		  setFunc = function(r,g,b,a)
						RengarusStatusDisplay.savedVariables.colorDodgingR = r
						RengarusStatusDisplay.savedVariables.colorDodgingG = g
						RengarusStatusDisplay.savedVariables.colorDodgingB = b
						RengarusStatusDisplay.savedVariables.colorDodgingA = a
					end,
		  width = "half",
		},
		[23] = {
		  type = "editbox",
		  name = "Text:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleDodging end,
		  default = RengarusStatusDisplay.Default.stringDodging,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringDodging end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringDodging = text end,
		  isMultiline = false,
		  width = "half",
		},
		--Blocking
		[24] = {
			type = "header",
			name = "Blocking",
		},
		[25] = {
			type = "checkbox",
			name = "Toggle:",
			tooltip = "Show when you are blocking.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleBlocking end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleBlocking = newValue end,
			width = "half",
		},
		[26] = {
		  type = "colorpicker",
		  name = "Color:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleBlocking end,
		  default = {r = RengarusStatusDisplay.Default.colorBlockingR,
					 g = RengarusStatusDisplay.Default.colorBlockingG,
					 b = RengarusStatusDisplay.Default.colorBlockingB,
					 a = RengarusStatusDisplay.Default.colorBlockingA},
		  getFunc = function() return
						RengarusStatusDisplay.savedVariables.colorBlockingR,
						RengarusStatusDisplay.savedVariables.colorBlockingG,
						RengarusStatusDisplay.savedVariables.colorBlockingB,
						RengarusStatusDisplay.savedVariables.colorBlockingA
					end,
		  setFunc = function(r,g,b,a)
						RengarusStatusDisplay.savedVariables.colorBlockingR = r
						RengarusStatusDisplay.savedVariables.colorBlockingG = g
						RengarusStatusDisplay.savedVariables.colorBlockingB = b
						RengarusStatusDisplay.savedVariables.colorBlockingA = a
					end,
		  width = "half",
		},
		[27] = {
		  type = "editbox",
		  name = "Text:",
		  default = RengarusStatusDisplay.Default.stringBlocking,
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleBlocking end,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringBlocking end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringBlocking = text end,
		  isMultiline = false,
		  width = "half",
		},
		--Sprinting
		[28] = {
			type = "header",
			name = "Sprinting",
		},
		[29] = {
			type = "checkbox",
			name = "Toggle:",
			tooltip = "Show when you are sprinting.\nWIP: Please let me know about false positives.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleSprinting end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleSprinting = newValue end,
			width = "half",
		},
		[30] = {
		  type = "colorpicker",
		  name = "Color:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleSprinting end,
		  default = {r = RengarusStatusDisplay.Default.colorSprintingR,
					 g = RengarusStatusDisplay.Default.colorSprintingG,
					 b = RengarusStatusDisplay.Default.colorSprintingB,
					 a = RengarusStatusDisplay.Default.colorSprintingA},
		  getFunc = function() return
						RengarusStatusDisplay.savedVariables.colorSprintingR,
						RengarusStatusDisplay.savedVariables.colorSprintingG,
						RengarusStatusDisplay.savedVariables.colorSprintingB,
						RengarusStatusDisplay.savedVariables.colorSprintingA
					end,
		  setFunc = function(r,g,b,a)
						RengarusStatusDisplay.savedVariables.colorSprintingR = r
						RengarusStatusDisplay.savedVariables.colorSprintingG = g
						RengarusStatusDisplay.savedVariables.colorSprintingB = b
						RengarusStatusDisplay.savedVariables.colorSprintingA = a
					end,
		  width = "half",
		},
		[31] = {
		  type = "editbox",
		  name = "Text:",
		  default = RengarusStatusDisplay.Default.stringSprinting,
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleSprinting end,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringSprinting end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringSprinting = text end,
		  isMultiline = false,
		  width = "half",
		},
		[32] = {
		  type = "editbox",
		  name = "Vampire Sprint:",
		  disabled = function() return not RengarusStatusDisplay.savedVariables.toggleSprinting end,
		  default = RengarusStatusDisplay.Default.stringVampireSprinting,
		  getFunc = function() return RengarusStatusDisplay.savedVariables.stringVampireSprinting end,
		  setFunc = function(text) RengarusStatusDisplay.savedVariables.stringVampireSprinting = text end,
		  isMultiline = false,
		  width = "half",
		},
		[33] = {
			type = "header",
			name = "Other",
		},
		[34] = {
			type = "checkbox",
			name = "Suppress Combat Tips",
			tooltip = "Supresses Active Combat Tips from showing up on screen.\nActive Combat Tips are needed to track certain effects.",
			default = true,
			getFunc = function() return RengarusStatusDisplay.savedVariables.toggleTips end,
			setFunc = function(newValue) RengarusStatusDisplay.savedVariables.toggleTips = newValue end,
			width = "half",
		},
		[35] = {
			type = "slider",
			name = "Scale",
			default = RengarusStatusDisplay.Default.scale,
			getFunc = function() return RengarusStatusDisplay.savedVariables.scale end,
			setFunc = function(value) RengarusStatusDisplay.savedVariables.scale = value UpdateAnchors() PreviewPosition() end,
			min = 0.1,
			max = 5,
			step = 0.1,
			width = "half"
		},
		[36] = {
			type = "slider",
			name = "X Position",
			default = RengarusStatusDisplay.Default.offsetX,
			getFunc = function() return RengarusStatusDisplay.savedVariables.offsetX end,
			setFunc = function(value) RengarusStatusDisplay.savedVariables.offsetX = value UpdateAnchors() PreviewPosition() end,
			min = -1000,
			max = 1000,
			step = 10,
			width = "half"
		},
		[37] = {
			type = "slider",
			name = "Y Position",
			default = RengarusStatusDisplay.Default.offsetY,
			getFunc = function() return RengarusStatusDisplay.savedVariables.offsetY end,
			setFunc = function(value) RengarusStatusDisplay.savedVariables.offsetY = value UpdateAnchors() PreviewPosition() end,
			min = -500,
			max = 500,
			step = 10,
			width = "half"
		},
	}
	LAM2:RegisterOptionControls("RengarusStatusDisplay_SettingsPanel", optionsData)
end

-----------
-- Setup --
-----------

local function OnAddOnLoaded(event, addonName)
	if addonName == RengarusStatusDisplay.name then
		EVENT_MANAGER:UnregisterForEvent(RengarusStatusDisplay.name, EVENT_ADD_ON_LOADED)
		EVENT_MANAGER:RegisterForEvent(RengarusStatusDisplay.name, EVENT_DISPLAY_ACTIVE_COMBAT_TIP, DisplayTip)
		EVENT_MANAGER:RegisterForEvent(RengarusStatusDisplay.name, EVENT_REMOVE_ACTIVE_COMBAT_TIP, RemoveTip)
		EVENT_MANAGER:RegisterForEvent(RengarusStatusDisplay.name, EVENT_EFFECT_CHANGED, OnEffectChanged)
		EVENT_MANAGER:RegisterForEvent(RengarusStatusDisplay.name, EVENT_COMBAT_EVENT, OnCombatEvent)
		EVENT_MANAGER:RegisterForEvent(RengarusStatusDisplay.name, EVENT_POWER_UPDATE, OnPowerUpdate)
		EVENT_MANAGER:RegisterForEvent(RengarusStatusDisplay.name, EVENT_ACTION_LAYER_POPPED, UpdateText)
		RengarusStatusDisplay.savedVariables = ZO_SavedVars:NewAccountWide("RengarusStatusDisplay_SavedVars", RengarusStatusDisplay.version, nil, RengarusStatusDisplay.Default)
		if LAM2 ~= false then              
			CreateSettingsWindow()
		end
		UpdateAnchors()
	end
end

EVENT_MANAGER:RegisterForEvent(RengarusStatusDisplay.name, EVENT_ADD_ON_LOADED, OnAddOnLoaded)